/*
 * Copyright © 2023 ESWIN.  All rights reserved.
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
 * all intellectual property and proprietary rights in and to this software.Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed,reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 * @Description: Common defination of video encode
 * @Author: ESWIN
 */

#ifndef __ES_COMM_VENC_H__
#define __ES_COMM_VENC_H__

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

#include "es_type.h"
#include "es_common.h"
#include "es_errno.h"
#include "es_comm_video.h"
#include "es_comm_rc.h"

typedef enum esVENC_ERR_CODE_E {
    VENC_ERR_FATAL = 20, /* unrecoverable error,such as hardware error */
    VENC_ERR_BUTT,
} VENC_ERR_CODE_E;

/* invlalid channel ID */
#define ES_ERR_VENC_INVALID_CHNID ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_CHNID)
/* at least one parameter is illegal, eg, an illegal enumeration value  */
#define ES_ERR_VENC_ILLEGAL_PARAM ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_ILLEGAL_PARAM)
/* channel exists */
#define ES_ERR_VENC_EXIST ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_EXIST)
/* channel unexists */
#define ES_ERR_VENC_UNEXIST ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_UNEXIST)
/* using a NULL pointer */
#define ES_ERR_VENC_NULL_PTR ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_NULL_PTR)
/* try to enable or initialize system, device or channel, before configing attribute */
#define ES_ERR_VENC_NOT_CONFIG ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_CONFIG)
/* operation is not supported by NOW */
#define ES_ERR_VENC_NOT_SUPPORT ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_SUPPORT)
/* operation is not permitted, eg, try to change static attribute */
#define ES_ERR_VENC_NOT_PERM ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_PERM)
/* failure caused by malloc memory */
#define ES_ERR_VENC_NOMEM ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOMEM)
/* failure caused by malloc buffer */
#define ES_ERR_VENC_NOBUF ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_NOBUF)
/* no data in buffer */
#define ES_ERR_VENC_BUF_EMPTY ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_EMPTY)
/* no buffer for new data */
#define ES_ERR_VENC_BUF_FULL ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_FULL)
/* system is not ready, had not initialized or loaded */
#define ES_ERR_VENC_SYS_NOTREADY ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_SYS_NOTREADY)
/* system is busy */
#define ES_ERR_VENC_BUSY ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, EN_ERR_BUSY)
/* unrecoverable error,such as hardware error */
#define ES_ERR_VENC_FATAL ES_DEF_ERR(ES_ID_VENC, EN_ERR_LEVEL_ERROR, VENC_ERR_FATAL)

#define ES_JPEG_ROI_RECT_NUM 32

/* the nalu type of h264e */
typedef enum esH264E_NALU_TYPE_E {
    H264E_NALU_BSLICE = 0,   /* B SLICE types */
    H264E_NALU_PSLICE = 1,   /* P SLICE types */
    H264E_NALU_ISLICE = 2,   /* I SLICE types */
    H264E_NALU_IDRSLICE = 5, /* IDR SLICE types */
    H264E_NALU_SEI = 6,      /* SEI types */
    H264E_NALU_SPS = 7,      /* SPS types */
    H264E_NALU_PPS = 8,      /* PPS types */
    H264E_NALU_BUTT
} H264E_NALU_TYPE_E;

/* the nalu type of h265e */
typedef enum esH265E_NALU_TYPE_E {
    H265E_NALU_BSLICE = 0,    /* B SLICE types */
    H265E_NALU_PSLICE = 1,    /* P SLICE types */
    H265E_NALU_ISLICE = 2,    /* I SLICE types */
    H265E_NALU_IDRSLICE = 19, /* IDR SLICE types */
    H265E_NALU_VPS = 32,      /* VPS types */
    H265E_NALU_SPS = 33,      /* SPS types */
    H265E_NALU_PPS = 34,      /* PPS types */
    H265E_NALU_SEI = 39,      /* SEI types */
    H265E_NALU_BUTT
} H265E_NALU_TYPE_E;

/* the pack type of jpege */
typedef enum esJPEGE_PACK_TYPE_E {
    JPEGE_PACK_ECS = 5,      /*ECS types*/
    JPEGE_PACK_APP = 6,      /*APP types*/
    JPEGE_PACK_VDO = 7,      /*VDO types*/
    JPEGE_PACK_PIC = 8,      /*PIC types*/
    JPEGE_PACK_DCF = 9,      /*DCF types*/
    JPEGE_PACK_DCF_PIC = 10, /*DCF PIC types*/
    JPEGE_PACK_BUTT
} JPEGE_PACK_TYPE_E;

/* the data type of VENC */
typedef union esVENC_DATA_TYPE_U {
    H264E_NALU_TYPE_E h264EType; /* R; H264E NALU types*/
    JPEGE_PACK_TYPE_E jpegEType; /* R; JPEGE pack types*/
    H265E_NALU_TYPE_E h265EType; /* R; H264E NALU types*/
} VENC_DATA_TYPE_U;

/*the pack info of VENC*/
typedef struct esVENC_PACK_INFO_S {
    VENC_DATA_TYPE_U packType; /* R; the pack type*/
    ES_U32 packOffset;
    ES_U32 packLength;
} VENC_PACK_INFO_S;

/* defines a stream packet */
typedef struct esVENC_PACK_S {
    ES_U64 phyAddr;         /* R; the physics address of stream */
    ES_U8 ATTRIBUTE* pAddr; /* R; the virtual address of stream */
    ES_U32 ATTRIBUTE len;   /* R; the length of stream */

    ES_U64 PTS;        /* R; PTS */
    ES_BOOL bFrameEnd; /* R; frame end */

    VENC_DATA_TYPE_U dataType;    /* R; the type of stream */
    ES_U32 offset;                /* R; the offset between the Valid data and the start address */
    ES_U32 dataNum;               /* R; the  stream packets num */
    VENC_PACK_INFO_S packInfo[8]; /* R; the stream packet Information */
} VENC_PACK_S;

/* defines the features of an H.264 stream */
typedef struct esVENC_STREAM_INFO_H264_S {
    ES_U32 picBytesNum;     /* R; the coded picture stream byte number */
    ES_U32 inter16x16MbNum; /* R; the inter16x16 macroblock num */
    ES_U32 inter8x8MbNum;   /* R; the inter8x8 macroblock num */
    ES_U32 intra16MbNum;    /* R; the intra16x16 macroblock num */
    ES_U32 intra8MbNum;     /* R; the intra8x8 macroblock num */
    ES_U32 intra4MbNum;     /* R; the inter4x4 macroblock num */

    ES_U32 updateAttrCnt; /* R; Number of times that channel attributes or parameters (including RC parameters) are
                                 set*/
    ES_U32 startQP;       /* R; the start QP of encoded frames*/
    ES_U32 meanQP;        /* R; the mean QP of encoded frames*/
    ES_BOOL bPSkip;
} VENC_STREAM_INFO_H264_S;

/* defines the features of an H.265 stream */
typedef struct esVENC_STREAM_INFO_H265_S {
    ES_U32 picBytesNum;     /* R; the coded picture stream byte number */
    ES_U32 inter64x64CuNum; /* R; the inter64x64 cu num */
    ES_U32 inter32x32CuNum; /* R; the inter32x32 cu num */
    ES_U32 inter16x16CuNum; /* R; the inter16x16 cu num */
    ES_U32 inter8x8CuNum;   /* R; the inter8x8   cu num */
    ES_U32 intra32x32CuNum; /* R; the Intra32x32 cu num */
    ES_U32 intra16x16CuNum; /* R; the Intra16x16 cu num */
    ES_U32 intra8x8CuNum;   /* R; the Intra8x8   cu num */
    ES_U32 intra4x4CuNum;   /* R; the Intra4x4   cu num */

    ES_U32 updateAttrCnt; /* R; Number of times that channel attributes or parameters (including RC parameters) are
                                 set */
    ES_U32 startQP;       /* R; the start QP of encoded frames */
    ES_U32 meanQP;        /* R; the mean QP of encoded frames */
    ES_BOOL bPSkip;
} VENC_STREAM_INFO_H265_S;

/* the sse info */
typedef struct esVENC_SSE_INFO_S {
    ES_BOOL bSSEEn; /* RW; Range:[0,1]; Region SSE enable */
    ES_U32 SSEVal;  /* R; Region SSE value */
} VENC_SSE_INFO_S;

/* the advance information of the h264e */
typedef struct esVENC_STREAM_ADVANCE_INFO_H264_S {
    ES_U32 residualBitNum;                  /* R; the residual num */
    ES_U32 MSELcuCnt;                       /* R; the lcu cnt of the mse */
    ES_U32 MSESum;                          /* R; the sum of the mse */
    VENC_SSE_INFO_S SSEInfo[8];             /* R; the information of the sse */
    ES_U32 QPHstgrm[ES_VENC_QP_HISGRM_NUM]; /* R; the QP histogram value */
    ES_U32 moveScene16x16Num;               /* R; the 16x16 cu num of the move scene */
    ES_U32 moveSceneBits;                   /* R; the stream bit num of the move scene */
} VENC_STREAM_ADVANCE_INFO_H264_S;

/* the advance information of the Jpege */
typedef struct esVENC_STREAM_ADVANCE_INFO_JPEG_S {
    // ES_U32 reserved;
} VENC_STREAM_ADVANCE_INFO_JPEG_S;

/* the advance information of the h265e */
typedef struct esVENC_STREAM_ADVANCE_INFO_H265_S {
    ES_U32 residualBitNum;                  /* R; the residual num */
    ES_U32 headBitNum;                      /* R; the head bit num */
    ES_U32 MADIVal;                         /* R; the madi value */
    ES_U32 MADPVal;                         /* R; the madp value */
    ES_DOUBLE dPSNRVal;                     /* R; the PSNR value */
    ES_U32 MSELcuCnt;                       /* R; the lcu cnt of the mse */
    ES_U32 MSESum;                          /* R; the sum of the mse */
    VENC_SSE_INFO_S SSEInfo[8];             /* R; the information of the sse */
    ES_U32 QPHstgrm[ES_VENC_QP_HISGRM_NUM]; /* R; the QP histogram value */
    ES_U32 moveScene32x32Num;               /* R; the 32x32 cu num of the move scene */
    ES_U32 moveSceneBits;                   /* R; the stream bit num of the move scene */
} VENC_STREAM_ADVANCE_INFO_H265_S;

/* defines the features of an jpege stream */
typedef struct esVENC_STREAM_INFO_JPEG_S {
    ES_U32 picBytesNum;   /* R; the coded picture stream byte number */
    ES_U32 updateAttrCnt; /* R; Number of times that channel attributes or parameters (including RC parameters) are
                                 set*/
    ES_U32 qFactor;       /* R; image quality */
} VENC_STREAM_INFO_JPEG_S;

/* defines the features of an stream */
typedef struct esVENC_STREAM_S {
    VENC_PACK_S ATTRIBUTE* pPack; /* R; stream pack attribute */
    ES_U32 ATTRIBUTE packCount;   /* R; the pack number of one frame stream */
    ES_U32 seq;                   /* R; the list number of stream */

    union {
        VENC_STREAM_INFO_H264_S h264Info; /* R; the stream info of h264 */
        VENC_STREAM_INFO_H265_S h265Info; /* R; the stream info of h265 */
        VENC_STREAM_INFO_JPEG_S jpegInfo; /* R; the stream info of jpeg */
    };

    union {
        VENC_STREAM_ADVANCE_INFO_H264_S advanceH264Info; /* R; the stream info of h264 */
        VENC_STREAM_ADVANCE_INFO_H265_S advanceH265Info; /* R; the stream info of h265 */
        VENC_STREAM_ADVANCE_INFO_JPEG_S advanceJpegInfo; /* R; the stream info of jpeg */
    };
} VENC_STREAM_S;

typedef struct esVENC_STREAM_INFO_S {
    ES_U32 picBytesNum; /* R; The coded picture stream byte number */
    ES_U32 picCnt;      /* R; Number of times that channel attributes or parameters (including RC parameters) are set */
    ES_U32 meanQP;      /* R; The mean QP of encoded frames */
    ES_BOOL bPSkip;

    ES_U32 residualBitNum; /* R; residual */
    ES_U32 MSESum;         /* R; Sum of MSE value */
    ES_U32 MSELcuCnt;      /* R; Sum of LCU number */
} VENC_STREAM_INFO_S;

typedef enum esVENC_BITDEPTH_E {
    BIT_DEPTH_DEFAULT = 0, /* the same with input data */
    BIT_DEPTH_8BIT,
    BIT_DEPTH_10BIT,
} BIT_DEPTH_E;

/* the attribute of h264e */
typedef struct esVENC_ATTR_H264_S {
    BIT_DEPTH_E bitDepth; /* RW; Range:[0, 2]; bitDepth of a picture to be encoded */
} VENC_ATTR_H264_S;

/* the attribute of h265e */
typedef struct esVENC_ATTR_H265_S {
    BIT_DEPTH_E bitDepth; /* RW; Range:[0, 2]; bitDepth of a picture to be encoded */
} VENC_ATTR_H265_S;

/* the profile of h264e */
typedef enum esPROFILE_H264 {
    PROFILE_H264_BASELINE,
    PROFILE_H264_MAIN,
    PROFILE_H264_HIGH,
    PROFILE_H264_HIGH10,
} PROFILE_H264_E;

/* the profile of h265e */
typedef enum esPROFILE_H265 {
    PROFILE_H265_MAIN,
    PROFILE_H265_MAIN10,
    PROFILE_H265_MAIN_STILL_PICTURE,
} PROFILE_H265_E;

/* the attribute of the Venc */
typedef struct esVENC_ATTR_S {
    PAYLOAD_TYPE_E type; /* RW; the type of payload */

    ES_U32 maxPicWidth;  /* RW; Range:[0,16384];maximum width of a picture to be encoded, in pixel */
    ES_U32 maxPicHeight; /* RW; Range:[0,16384];maximum height of a picture to be encoded, in pixel */

    ES_U32 bufSize;   /* RW; stream buffer size */
    ES_U32 profile;   /* RW; H.264: 0:baseline; 1:MP; 2:HP; 3:HP10 [0,3];
                             H.265: 0:MP; 1:Main10; 2:Main Still Pic; [0,2];
                             Jpege/MJpege: 0:Baseline */
    ES_BOOL bByFrame; /* RW; Range:[0,1]; get stream mode is slice mode or frame mode */
    ES_U32 picWidth;  /* RW; Range:[0,16384];width of a picture to be encoded, in pixel */
    ES_U32 picHeight; /* RW; Range:[0,16384];height of a picture to be encoded, in pixel */
    ES_U32 align;     /* RW; Align works with stride in VIDEO_FRAME_S*/

    PIXEL_FORMAT_E pixelFormat; /* RW; Pixel format of a video picture */
    union {
        VENC_ATTR_H264_S attrH264e; /* attributes of H264e */
        VENC_ATTR_H265_S attrH265e; /* attributes of H265e */
    };
} VENC_ATTR_S;

/* the GOP mode */
typedef enum esVENC_GOP_MODE_E {
    VENC_GOPMODE_NORMALP = 0, /* NORMALP */
    VENC_GOPMODE_DUALREF,     /* DUALREF */
    VENC_GOPMODE_SMARTREF,    /* SMARTREF */
    VENC_GOPMODE_ADVSMARTREF, /* ADVSMARTREF */
    VENC_GOPMODE_BIPREDB,     /* BIPREDB: P can only ref one */
    VENC_GOPMODE_LOWDELAYB,   /* LOWDELAYB */
    VENC_GOPMODE_BUTT,
} VENC_GOP_MODE_E;

/* the attribute of the normalp */
typedef struct esVENC_GOP_NORMALP_S {
    ES_S32 IPQpDelta; /* RW; Range:[-51,51]; QP variance between P frame and I frame */
} VENC_GOP_NORMALP_S;

/* the attribute of the dualref */
typedef struct esVENC_GOP_DUALREF_S {
    ES_U32 SBInterval; /* RW; Range:[0, 1) U (1, gop); Interval of the special B frames, should be less than Gop */
    ES_S32 SBQpDelta;  /* RW; Range:[-51,51]; QP variance between general P/B frame and special B frame */
    ES_S32 IPBQpDelta; /* RW; Range:[-51,51]; QP variance between general P/B frame and I frame */
} VENC_GOP_DUALREF_S;

/* the attribute of the smartref */
typedef struct esVENC_GOP_SMARTREF_S {
    ES_U32 BgInterval; /* RW; Interval of the long-term reference frame, can not be less than gop */
    ES_S32 BgQpDelta;  /* RW; Range:[-51,51]; QP variance between P frame and Bg frame */
    ES_S32 ViQpDelta;  /* RW; Range:[-51,51]; QP variance between P frame and virtual I  frame */
} VENC_GOP_SMARTREF_S;

/* the attribute of the advsmartref */
typedef struct esVENC_GOP_ADVSMARTREF_S {
    ES_U32 BgInterval; /* RW; Interval of the long-term reference frame, can not be less than gop + 1*/
    ES_S32 BgQpDelta;  /* RW; Range:[-51,51]; QP variance between P frame and Bg frame */
    ES_S32 ViQpDelta;  /* RW; Range:[-51,51]; QP variance between P frame and virtual I  frame */
} VENC_GOP_ADVSMARTREF_S;

/* the attribute of the bipredb */
typedef struct esVENC_GOP_BIPREDB_S {
    ES_U32 BFrmNum;   /* RW; Range:[1,3]; Number of B frames */
    ES_S32 BQpDelta;  /* RW; Range:[-51,51]; QP variance between P frame and B frame */
    ES_S32 IPQpDelta; /* RW; Range:[-51,51]; QP variance between P frame and I frame */
} VENC_GOP_BIPREDB_S;

/* the attribute of the lowdelay b */
typedef struct esVENC_GOP_LOWDELAYB_S {
    ES_U32 BFrmNum;  /* RW; Range:[1,4]; Number of B frames */
    ES_S32 IQpDelta; /* RW; Range:[-51,51]; QP variance between other frame and I frame */
} VENC_GOP_LOWDELAYB_S;

/* the attribute of the GOP */
typedef struct esVENC_GOP_ATTR_S {
    VENC_GOP_MODE_E GOPMode; /* RW; Encoding GOP type */
    union {
        VENC_GOP_NORMALP_S normalP;         /* attributes of normal P */
        VENC_GOP_DUALREF_S dualRef;         /* attributes of dual P*/
        VENC_GOP_SMARTREF_S smartRef;       /* attributes of Smart P */
        VENC_GOP_ADVSMARTREF_S advSmartRef; /* attributes of AdvSmart P */
        VENC_GOP_BIPREDB_S bipredB;         /* attributes of bipredb */
        VENC_GOP_LOWDELAYB_S lowdelayB;     /* attributes of lowdelayb */
    };
} VENC_GOP_ATTR_S;

/* the attribute of the venc channel */
typedef struct esVENC_CHN_ATTR_S {
    VENC_ATTR_S vencAttr;    /*the attribute of video encoder */
    VENC_RC_ATTR_S rcAttr;   /*the attribute of rate ctrl */
    VENC_GOP_ATTR_S GOPAttr; /*the attribute of GOP */
} VENC_CHN_ATTR_S;

/* the param of receive picture */
typedef struct esVENC_RECV_PIC_PARAM_S {
    ES_S32 recvPicNum; /* RW; Range:[-1, 2147483647]; Number of frames received and encoded by the encoding
                              channel, 0 is not supported*/
} VENC_RECV_PIC_PARAM_S;

typedef struct esVENC_ENCODE_ERROR_S {
    ES_S32 vencInnerErr;        /* R; the encoder inner error */
    ES_S32 vencResetErr;        /* R; the encoder hardware is reset */
    ES_S32 vencFrameNotRelease; /* R; the frame not released for too long time */
    ES_S32 memErrSet;           /* R; the memory is not enough */
    ES_S32 vencHardwareErr;     /* R: hardware encountered an unrecoverable error */
} VENC_ENCODE_ERROR_S;

/* the status of the venc channel */
typedef struct esVENC_CHN_STATUS_S {
    ES_U32 leftPics;         /* R; left picture number */
    ES_U32 leftStreamBytes;  /* R; left stream bytes */
    ES_U32 leftStreamFrames; /* R; left stream frames */
    ES_U32 curPacks;         /* R; pack number of current frame */
    ES_U32 leftRecvPics; /* R; number of frames to be received. This member is valid after ES_MPI_VENC_StartRecvPicEx is
                            called. */
    ES_U32 leftEncPics;  /* R; number of frames to be encoded. This member is valid after ES_MPI_VENC_StartRecvPicEx is
                            called.*/
    ES_BOOL bStartRecvFrame;        /* R; had started recv frame */
    VENC_ENCODE_ERROR_S vencEncErr; /* R; information about encoder error */
    VENC_STREAM_INFO_S vencStrmInfo;
} VENC_CHN_STATUS_S;

/* the param of the h264e slice split */
typedef struct esVENC_H264_SLICE_SPLIT_S {
    ES_BOOL bSplitEnable; /* RW; Range:[0,1]; slice split enable, default: ES_FALSE */
    ES_U32 MBLineNum;     /* RW; the max number is (Picture height + 15)/16; this value presents the mb line number of
                                 one slice*/
} VENC_H264_SLICE_SPLIT_S;

/* the param of the h264e intra pred */
typedef struct esVENC_H264_INTRA_PRED_S {
    ES_U32 intraPredFlag; /* RW; Range:[0,1]; default: ES_FALSE, see the H.264 protocol for the meaning */
} VENC_H264_INTRA_PRED_S;

/* the param of the h264e trans */
typedef struct esVENC_H264_TRANS_S {
    ES_S32 chromaQPIndexOffset; /* RW; Range:[-12,12];default value: 0, see the H.264 protocol for the meaning */
} VENC_H264_TRANS_S;

/* the param of the h264e entropy */
typedef struct esVENC_H264_ENTROPY_S {
    ES_U32 entropyEncMode; /* RW; Range:[0,1]; Entropy encoding mode, 0:cavlc, 1:cabac */
} VENC_H264_ENTROPY_S;

/* the param of the h264e deblocking */
typedef struct esVENC_H264_DBLK_S {
    ES_U32 disDeblkFilterIdc; /*  RW; Range:[0,1]; see the H.264 protocol for the meaning */
    ES_S32 sliceAlphaOffset;  /*  RW; Range:[-6,6]; see the H.264 protocol for the meaning */
    ES_S32 sliceBetaOffset;   /*  RW; Range:[-6,6]; see the H.264 protocol for the meaning */
} VENC_H264_DBLK_S;

/* the param of the h264e VUI timing info */
typedef struct esVENC_H264_VUI_TIME_INFO_S {
    ES_U8 timingInfoPresentFlag; /* RW; Range:[0,1]; If 1, timing info belows will be encoded into VUI. */
    ES_U8 fixedFrameRateFlag;    /* RW; Range:[0,1]; see the H.264 protocol for the meaning. */
    ES_U32 numUnitsInTick;       /* RW; Range:(0,4294967295]; see the H.264 protocol for the meaning */
    ES_U32 timeScale;            /* RW; Range:(0,4294967295]; see the H.264 protocol for the meaning */
} VENC_VUI_H264_TIME_INFO_S;

/* the param of the VUI aspect ratio */
typedef struct esVENC_VUI_ASPECT_RATIO_S {
    ES_U8 aspectRatioInfoPresentFlag; /* RW; Range:[0,1]; If 1, aspectratio info belows will be encoded into VUI */
    ES_U8 aspectRatioIdc;             /* RW; Range:[0,255]; 17~254 is reserved,see the protocol for the meaning. */
    ES_U8 overscanInfoPresentFlag;    /* RW; Range:[0,1]; If 1, oversacan info belows will be encoded into VUI. */
    ES_U8 overscanAppropriateFlag;    /* RW; Range:[0,1]; see the protocol for the meaning. */
    ES_U16 sarWidth;                  /* RW; Range:(0, 65535]; see the protocol for the meaning. */
    ES_U16 sarHeight;                 /* RW; Range:(0, 65535]; see the protocol for the meaning.
                                         notes: sar_width  and  sarHeight  shall  be  relatively  prime. */
} VENC_VUI_ASPECT_RATIO_S;

/* the param of the VUI video signal */
typedef struct esVENC_VUI_VIDEO_SIGNAL_S {
    ES_U8 videoSignalTypePresentFlag; /* RW; Range:[0,1]; If 1, video singnal info will be encoded into VUI. */
    ES_U8 videoFormat;        /* RW; H.264e Range:[0,7], H.265e Range:[0,5]; see the protocol for the meaning. */
    ES_U8 videoFullRangeFlag; /* RW; Range: [0,1]; see the protocol for the meaning. */
    ES_U8 colourDescriptionPresentFlag; /* RO; Range: [0,1]; see the protocol for the meaning. */
    ES_U8 colourPrimaries;              /* RO; Range: [0,255]; see the protocol for the meaning. */
    ES_U8 transferCharacteristics;      /* RO; Range: [0,255]; see the protocol for the meaning. */
    ES_U8 matrixCoefficients;           /* RO; Range:[0,255]; see the protocol for the meaning. */
} VENC_VUI_VIDEO_SIGNAL_S;

/* the param of the VUI video signal */
typedef struct esVENC_VUI_BITSTREAM_RESTRIC_S {
    ES_U8 bitstreamRestrictionFlag; /* RW; Range: [0,1]; see the protocol for the meaning. */
} VENC_VUI_BITSTREAM_RESTRIC_S;

/* the param of the h264e VUI */
typedef struct esVENC_H264_VUI_S {
    VENC_VUI_ASPECT_RATIO_S VUIAspectRatio;
    VENC_VUI_H264_TIME_INFO_S VUITimeInfo;
    VENC_VUI_VIDEO_SIGNAL_S VUIVideoSignal;
    VENC_VUI_BITSTREAM_RESTRIC_S VUIBitstreamRestric;
} VENC_H264_VUI_S;

/* the param of the h265e VUI timing info */
typedef struct esVENC_VUI_H265_TIME_INFO_S {
    ES_U32 timingInfoPresentFlag;    /* RW; Range:[0,1]; If 1, timing info belows will be encoded into VUI.*/
    ES_U32 numUnitsInTick;           /* RW; Range:[0,4294967295]; see the H.265 protocol for the meaning. */
    ES_U32 timeScale;                /* RW; Range:(0,4294967295]; see the H.265 protocol for the meaning */
    ES_U32 numTicksPocDiffOneMinus1; /* RW; Range:(0,4294967294]; see the H.265 protocol for the meaning */
} VENC_VUI_H265_TIME_INFO_S;

/* the param of the h265e VUI */
typedef struct esVENC_H265_VUI_S {
    VENC_VUI_ASPECT_RATIO_S VUIAspectRatio;
    VENC_VUI_H265_TIME_INFO_S VUITimeInfo;
    VENC_VUI_VIDEO_SIGNAL_S VUIVideoSignal;
    VENC_VUI_BITSTREAM_RESTRIC_S VUIBitstreamRestric;
} VENC_H265_VUI_S;

/* the param of the jpege */
typedef struct esVENC_JPEG_PARAM_S {
    ES_U32 qFactor;   /* RW; Range:[1,99]; Qfactor value  */
    ES_U8 yQt[64];    /* RW; Range:[1, 255]; Y quantization table */
    ES_U8 cQt[64];    /* RW; Range:[1, 255]; C quantization table */
    ES_U32 MCUPerECS; /* RW; the max MCU row number is (picheight +15) >> 4; MCU row number of one
                         ECS*/
} VENC_JPEG_PARAM_S;

/* the param of the mjpege */
typedef struct esVENC_MJPEG_PARAM_S {
    ES_U8 yQt[64];    /* RW; Range:[1, 255]; Y quantization table */
    ES_U8 cQt[64];    /* RW; Range:[1, 255]; C quantization table */
    ES_U32 MCUPerECS; /* RW; the max MCU row number is (picheight +15) >> 4; MCU row number of one
                         ECS*/
} VENC_MJPEG_PARAM_S;

/* the attribute of the roi */
typedef struct esVENC_ROI_ATTR_S {
    ES_U32 index;    /* RW; Range:[0, 7]; Index of an ROI. The system supports indexes ranging from 0 to 7 */
    ES_BOOL bEnable; /* RW; Range:[0, 1]; Whether to enable this ROI */
    ES_BOOL bAbsQP;  /* RW; Range:[0, 1]; QP mode of an ROI.ES_FALSE: relative QP.ES_TURE: absolute QP.*/
    ES_S32 QP;       /* RW; Range:[-51, 51]; QP value,only relative mode can QP value less than 0. */
    RECT_S rect;     /* RW; Region of an ROI*/
} VENC_ROI_ATTR_S;

typedef struct esVENC_JPEG_ROI_ATTR_S {
    RECT_S ROIMap[ES_JPEG_ROI_RECT_NUM]; /* Region of non ROI*/
    ES_U32 ROIRectNum;                   /* ROI region number*/
    ES_U8 nonROILumaFilter[64];          /* non ROI region quantization tables for luma filter. */
    ES_U8 nonROIChromaFilter[64];        /* non ROI region quantization tables for chroma filter. */
    ES_S32 nonROILevel;                  /* RW; Range:[0, 9]; non ROI region qLevel. */
} VENC_JPEG_ROI_ATTR_S;

/* the param of the advanced skip frame */
typedef struct esVENC_REF_PARAM_S {
    ES_U32 base;        /* RW; Range:[0,4294967295]; Base layer period*/
    ES_U32 enhance;     /* RW; Range:[0,255]; Enhance layer period*/
    ES_BOOL enablePred; /* RW; Range:[0, 1]; Whether some frames at the base layer are referenced by other frames at
    the base layer. When bEnablePred is ES_FALSE, all frames at the base layer reference IDR frames.*/
} VENC_REF_PARAM_S;

/* the information of the stream */
typedef struct esVENC_STREAM_BUF_INFO_S {
    ES_U64 phyAddr[ES_MAX_TILE_NUM];               /* R; Start physical address for a stream buffer */
    ES_VOID ATTRIBUTE* pUserAddr[ES_MAX_TILE_NUM]; /* R; Start virtual address for a stream buffer */
    ES_U64 ATTRIBUTE bufSize[ES_MAX_TILE_NUM];     /* R; Stream buffer size */
} VENC_STREAM_BUF_INFO_S;

/* the param of the h265e slice split */
typedef struct esVENC_H265_SLICE_SPLIT_S {
    ES_BOOL bSplitEnable; /* RW; Range:[0,1]; slice split enable, default:ES_FALSE */
    ES_U32 lcuLineNum;    /* RW; Range:(Picture height + lcu size minus one)/lcu size; This value presents lcu line
                                 number */
} VENC_H265_SLICE_SPLIT_S;

/* the param of the h265e pu */
typedef struct esVENC_H265_PU_S {
    ES_U32 constrainedIntraPredFlag;        /* RW; Range:[0,1]; see the H.265 protocol for the meaning. */
    ES_U32 strongIntraSmoothingEnabledFlag; /* RW; Range:[0,1]; see the H.265 protocol for the meaning. */
} VENC_H265_PU_S;

/* the param of the h265e trans */
typedef struct esVENC_H265_TRANS_S {
    ES_S32 cbQPOffset; /* RW; Range:[-12,12]; see the H.265 protocol for the meaning. */
    ES_S32 crQPOffset; /* RW; Range:[-12,12]; see the H.265 protocol for the meaning. */

    ES_BOOL bScalingListEnabled; /* RW; Range:[0,1]; If 1, specifies that a scaling list is used.*/
} VENC_H265_TRANS_S;

/* the param of the h265e deblocking */
typedef struct esVENC_H265_DBLK_S {
    ES_U32 sliceDeblkFilterDisabledFlag; /* RW; Range:[0,1]; see the H.265 protocol for the meaning. */
    ES_S32 sliceBetaOffset;              /* RW; Range:[-6,6]; see the H.265 protocol for the meaning. */
    ES_S32 sliceTcOffset;                /* RW; Range:[-6,6]; see the H.265 protocol for the meaning. */
} VENC_H265_DBLK_S;

/* the param of the h265e SAO */
typedef struct esVENC_H265_SAO_S {
    ES_U32 sliceSAOLumaFlag;   /*RW; Range:[0,1]; Indicates whether SAO filtering is performed on the luminance
                                      component of the current slice. */
    ES_U32 sliceSAOChromaFlag; /*RW; Range:[0,1]; Indicates whether SAO filtering is performed on the chrominance
                                      component of the current slice*/
} VENC_H265_SAO_S;

/* venc mode type */
typedef enum esVENC_INTRA_REFRESH_MODE_E {
    INTRA_REFRESH_ROW = 0, /* Line mode */
    INTRA_REFRESH_COLUMN,  /* Column mode */
    INTRA_REFRESH_BUTT
} VENC_INTRA_REFRESH_MODE_E;

/* the param of the intra refresh */
typedef struct esVENC_INTRA_REFRESH_S {
    ES_BOOL bRefreshEnable; /*RW; Range:[0,1]; intra refresh enable, ES_TRUE:enable, ES_FALSE:diable, default
                                value:ES_FALSE*/
    VENC_INTRA_REFRESH_MODE_E intraRefreshMode; /*RW; The mode of intra refresh*/
    ES_U32 refreshNum; /*RW; Number of rows/column to be refreshed during each I macroblock refresh*/
    ES_U32 reqIQP;     /*RW; Range:[0,51]; QP value of the I frame*/
} VENC_INTRA_REFRESH_S;

/* venc mode type */
typedef enum esVENC_MODTYPE_E {
    MODTYPE_VENC = 1, /* VENC */
    MODTYPE_H264E,    /* H264e */
    MODTYPE_H265E,    /* H265e */
    MODTYPE_JPEGE,    /* Jpege */
    MODTYPE_BUTT
} VENC_MODTYPE_E;

/* the param of the h264e mod */
typedef struct esVENC_MOD_H264E_S {
    ES_U32 oneStreamBuffer;    /* RW; Range:[0,1]; one stream buffer*/
    ES_U32 h264eMiniBufMode;   /* RW; Range:[0,1]; H264e MiniBufMode*/
    VB_SOURCE_E h264eVBSource; /* RW; H264e VBSource*/
    ES_BOOL bQPHstgrmEn;       /* RW; Range:[0,1]*/
} VENC_MOD_H264E_S;

/* the param of the h265e mod */
typedef struct esVENC_MOD_H265E_S {
    ES_U32 oneStreamBuffer;    /* RW; Range:[0,1]; one stream buffer*/
    ES_U32 h265eMiniBufMode;   /* RW; Range:[0,1]; H265e MiniBufMode*/
    VB_SOURCE_E h265eVBSource; /* RW; H265e VBSource*/
    ES_BOOL bQPHstgrmEn;       /* RW; Range:[0,1]*/
} VENC_MOD_H265E_S;

/* the param of the jpege mod */
typedef struct esVENC_MOD_JPEGE_S {
    ES_U32 jpegeMiniBufMode;   /* RW; Range:[0,1]; Jpege MiniBufMode*/
    ES_U32 jpegClearStreamBuf; /* RW; Range:[0,1]; JpegClearStreamBuf*/
} VENC_MOD_JPEGE_S;

/* the param of the venc mod */
typedef struct esVENC_MOD_VENC_S {
    // Reserved;
} VENC_MOD_VENC_S;

/* the param of the mod */
typedef struct esVENC_MODPARAM_S {
    VENC_MODTYPE_E vencModType; /* RW; VencModType*/
    union {
        VENC_MOD_VENC_S vencModParam;
        VENC_MOD_H264E_S h264eModParam;
        VENC_MOD_H265E_S h265eModParam;
        VENC_MOD_JPEGE_S jpegeModParam;
    };
} VENC_PARAM_MOD_S;

typedef enum esVENC_FRAME_TYPE_E {
    VENC_FRAME_TYPE_NONE = 1,
    VENC_FRAME_TYPE_IDR,
    VENC_FRAME_TYPE_I,
    VENC_FRAME_TYPE_P,
    VENC_FRAME_TYPE_B,
    VENC_FRAME_TYPE_BUTT
} VENC_FRAME_TYPE_E;

/* the information of the user rc */
typedef struct esUSER_RC_INFO_S {
    ES_BOOL bQPMapValid;         /* RW; Range:[0,1]; Indicates whether the QPMap mode is valid for the current frame*/
    ES_BOOL bMetaMapValid;       /* RW; Range:[0,1]; Indicates whether the metamap is valid for the current frame*/
    ES_U32 blkStartQP;           /* RW; Range:[0,51];QP value of the first 16 x 16 block in QPMap mode */
    ES_U64 qpMapFd;              /* RW; Mem fd of the QP table in QPMap mode*/
    ES_U64 metaMapFd;            /* RW; Mem fd of the meta table in QPMap mode per qpMapMeta*/
    VENC_FRAME_TYPE_E frameType; /* RW; Encoding frame type of the current frame */
} USER_RC_INFO_S;

/* the information of the user frame */
typedef struct esUSER_FRAME_INFO_S {
    VIDEO_FRAME_INFO_S userFrame;
    USER_RC_INFO_S userRcInfo;
} USER_FRAME_INFO_S;

/* the config of the sse */
typedef struct esVENC_SSE_CFG_S {
    ES_U32 index;    /* RW; Range:[0, 7]; Index of an SSE. The system supports indexes ranging from 0 to 7 */
    ES_BOOL bEnable; /* RW; Range:[0, 1]; Whether to enable SSE */
    RECT_S rect;     /* RW; */
} VENC_SSE_CFG_S;

/* the param of the crop */
typedef struct esVENC_CROP_INFO_S {
    ES_BOOL bEnable; /* RW; Range:[0, 1]; Crop region enable */
    RECT_S rect;     /* RW; Crop region, note: x must be multi of 16 */
} VENC_CROP_INFO_S;

/* the param of the venc frame rate */
typedef struct esVENC_FRAME_RATE_S {
    ES_S32 srcFrmRate; /* RW; Range: -1 or [1, 240]; Input frame rate of a channel */
    ES_S32 dstFrmRate; /* RW; Range: -1 or [1, 240]; Output frame rate of a channel */
} VENC_FRAME_RATE_S;

typedef struct esVENC_CONST_CHROMA_S {
    ES_BOOL bEnableConstChroma; /* RW; Range:[0, 1]; Whether to enable const chroma. */
    ES_U32 cbValue;             /* RW; 8Bit Range:[0, 255], 10Bit Range:[0, 1023]; The constant pixel value for Cb. */
    ES_U32 crValue;             /* RW; 8Bit Range:[0, 255], 10Bit Range:[0, 1023]; The constant pixel value for Cr. */
} VENC_CONST_CHROMA_S;

/* the param of the venc encode channel */
typedef struct esVENC_CHN_PARAM_S {
    ES_U32 priority;                 /* RW; Range:[0, 1]; The priority of the coding chnl. */
    ES_U32 maxStrmCnt;               /* RW: Range:[0,4294967295]; Maximum number of frames in a stream buffer */
    ES_U32 pollWakeUpFrmCnt;         /* RW: Range:(0,4294967295]; the frame num needed to wake up obtaining streams */
    VENC_FRAME_RATE_S frameRate;     /* RW: Defines the frame rate control parameters of a VENC chnl. */
    VENC_CONST_CHROMA_S constChroma; /* RW: Defines the const chroma, eg, gray */
    VENC_CROP_INFO_S cropCfg;        /* RW: Crop config */
    ROTATION_E rotation;             /* RW: Rotation config */
    ES_BOOL bEnableCUInfo;           /* RW: Range:[0, 1]; Disable or enable CU info in getStream api; Default: 0 */
} VENC_CHN_PARAM_S;

typedef struct esVENC_CU_PREDICTION_S {
    OPERATION_MODE_E predMode; /* RW; CU tendency configuration mode  */

    ES_U32 intra32Cost; /* RW; Range:[0,15]; Tendency adjustment in Intra32 mode*/
    ES_U32 intra16Cost; /* RW; Range:[0,15]; Tendency adjustment in Intra16 mode*/
    ES_U32 intra8Cost;  /* RW; Range:[0,15]; Tendency adjustment in Intra8 mode*/
    ES_U32 intra4Cost;  /* RW; Range:[0,15]; Tendency adjustment in Intra4 mode*/

    ES_U32 inter64Cost; /* RW; Range:[0,15]; Tendency adjustment in Intra64 mode*/
    ES_U32 inter32Cost; /* RW; Range:[0,15]; Tendency adjustment in Inter32 mode*/
    ES_U32 inter16Cost; /* RW; Range:[0,15]; Tendency adjustment in Inter16 mode*/
    ES_U32 inter8Cost;  /* RW; Range:[0,15]; Tendency adjustment in Inter8 mode*/
} VENC_CU_PREDICTION_S;

typedef struct esVENC_SKIP_BIAS_S {
    ES_BOOL bSkipBiasEn;       /* RW; Range:[0,1];   Flag indicating whether the skip tendency function is enabled*/
    ES_U32 skipThreshGain;     /* RW; Range:[0,15];  used to calculate the SAD threshold for foreground detection*/
    ES_U32 skipThreshOffset;   /* RW; Range:[0,255]; used to calculate the SAD threshold for foreground detection*/
    ES_U32 skipBackgroundCost; /* RW; Range:[0,15];  Skip tendency adjustment in the background*/
    ES_U32 skipForegroundCost; /* RW; Range:[0,15];  Skip tendency adjustment in the foreground*/
} VENC_SKIP_BIAS_S;

typedef struct esVENC_CHN_POOL_S {
    VB_POOL picVbPool;     /* RW;  vb pool id for pic buffer */
    VB_POOL picInfoVbPool; /* RW;  vb pool id for pic info buffer */
} VENC_CHN_POOL_S;

typedef struct esVENC_RC_ADVPARAM_S {
    ES_U32 clearStatAfterSetAttr; /* RW; Range:[0,1]; Clear Stat After SetAttr enable */
} VENC_RC_ADVPARAM_S;

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif /* __ES_COMM_VENC_H__ */
