/*
 * Copyright © 2023 ESWIN. All rights reserved.
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
 * all intellectual property and proprietary rights in and to this software.Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed,reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 * @Description: Define all the data stuctures.
 * @Author: ESWIN
 */

#ifndef __ES_COMM_VO_H__
#define __ES_COMM_VO_H__

#include "es_type.h"
#include "es_errno.h"
#include "es_common.h"
#include "es_comm_video.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* End of #ifdef __cplusplus */

typedef enum esEN_VOU_ERR_CODE_E {
    EN_ERR_VO_DEV_NOT_CONFIG = 0x40,
    EN_ERR_VO_DEV_NOT_ENABLE = 0x41,
    EN_ERR_VO_DEV_HAS_ENABLED = 0x42,
    EN_ERR_VO_DEV_HAS_BINDED = 0x43,
    EN_ERR_VO_DEV_NOT_BINDED = 0x44,

    ERR_VO_NOT_ENABLE = 0x45,
    ERR_VO_NOT_DISABLE = 0x46,
    ERR_VO_NOT_CONFIG = 0x47,

    ERR_VO_CHN_NOT_DISABLE = 0x48,
    ERR_VO_CHN_NOT_ENABLE = 0x49,
    ERR_VO_CHN_NOT_CONFIG = 0x4a,

    ERR_VO_WAIT_TIMEOUT = 0x4e,
    ERR_VO_INVALID_VFRAME = 0x4f,
    ERR_VO_INVALID_RECT_PARA = 0x50,
    ERR_VO_SETBEGIN_ALREADY = 0x51,
    ERR_VO_SETBEGIN_NOTYET = 0x52,
    ERR_VO_SETEND_ALREADY = 0x53,
    ERR_VO_SETEND_NOTYET = 0x54,

    ERR_VO_WBC_NOT_DISABLE = 0x69,
    ERR_VO_WBC_NOT_CONFIG = 0x6a,

    ERR_VO_CHN_AREA_OVERLAP = 0x6b,

    EN_ERR_INVALID_WBCID = 0x6c,
    EN_ERR_INVALID_LAYERID = 0x6d,
    EN_ERR_VO_VIDEO_HAS_BINDED = 0x6e,
    EN_ERR_VO_VIDEO_NOT_BINDED = 0x6f,
    ERR_VO_WBC_HAS_BIND = 0x70,
    ERR_VO_WBC_HAS_CONFIG = 0x71,
    ERR_VO_WBC_NOT_BIND = 0x72,
    ERR_VO_WBC_NOT_ENABLE = 0x73,

    /* HDMI */
    ERR_HDMI_INVALID_ID = 0x90,
    ERR_HDMI_NOT_INIT = 0x91,
    ERR_HDMI_INVALID_PARAM = 0x92,
    ERR_HDMI_NULL_PTR = 0x93,
    ERR_HDMI_NOT_CONNECT = 0x94,
    ERR_HDMI_READ_SINK_FAILED = 0x95,
    ERR_HDMI_INIT_ALREADY = 0x96,
    ERR_HDMI_CALLBACK_ALREADY = 0x97,
    ERR_HDMI_INVALID_CALLBACK = 0x98,
    ERR_HDMI_BUS_BUSY = 0x99,
    ERR_HDMI_MALLOC_FAILED = 0x9A,
    ERR_HDMI_FEATURE_NO_SUPPORT = 0x9B,

    /* MIPI */
    ERR_MIPI_INVALID_ID = 0x100,
    ERR_MIPI_NOT_INIT = 0x101,
    ERR_MIPI_INVALID_PARAM = 0x102,
    ERR_MIPI_NULL_PTR = 0x103,
    ERR_MIPI_NOT_CONNECT = 0x104,

    /* new added */
    ERR_VO_BUTT

} EN_VOU_ERR_CODE_E;

/* System define error code */
#define ES_ERR_VO_BUSY ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_BUSY)
#define ES_ERR_VO_NO_MEM ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_NOMEM)
#define ES_ERR_VO_NULL_PTR ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_NULL_PTR)
#define ES_ERR_VO_SYS_NOTREADY ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_SYS_NOTREADY)
#define ES_ERR_VO_INVALID_DEVID ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_DEVID)
#define ES_ERR_VO_INVALID_CHNID ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_CHNID)
#define ES_ERR_VO_ILLEGAL_PARAM ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_ILLEGAL_PARAM)
#define ES_ERR_VO_NOT_SUPPORT ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_SUPPORT)
#define ES_ERR_VO_NOT_PERMIT ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_PERM)
#define ES_ERR_VO_INVALID_WBCID ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_WBCID)
#define ES_ERR_VO_INVALID_LAYERID ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_LAYERID)

/* Device relative error code */
#define ES_ERR_VO_DEV_NOT_CONFIG ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_VO_DEV_NOT_CONFIG)
#define ES_ERR_VO_DEV_NOT_ENABLE ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_VO_DEV_NOT_ENABLE)
#define ES_ERR_VO_DEV_HAS_ENABLED ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_VO_DEV_HAS_ENABLED)
#define ES_ERR_VO_DEV_HAS_BINDED ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_VO_DEV_HAS_BINDED)
#define ES_ERR_VO_DEV_NOT_BINDED ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_VO_DEV_NOT_BINDED)

/* Video layer relative error code */
#define ES_ERR_VO_VIDEO_NOT_ENABLE ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_NOT_ENABLE)
#define ES_ERR_VO_VIDEO_NOT_DISABLE ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_NOT_DISABLE)
#define ES_ERR_VO_VIDEO_NOT_CONFIG ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_NOT_CONFIG)
#define ES_ERR_VO_VIDEO_HAS_BINDED ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_VO_VIDEO_HAS_BINDED)
#define ES_ERR_VO_VIDEO_NOT_BINDED ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, EN_ERR_VO_VIDEO_NOT_BINDED)

/*WBC Relative error code*/
#define ES_ERR_VO_WBC_NOT_DISABLE ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_WBC_NOT_DISABLE)
#define ES_ERR_VO_WBC_NOT_CONFIG ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_WBC_NOT_CONFIG)
#define ES_ERR_VO_WBC_HAS_CONFIG ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_WBC_HAS_CONFIG)
#define ES_ERR_VO_WBC_NOT_BIND ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_WBC_NOT_BIND)
#define ES_ERR_VO_WBC_HAS_BIND ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_WBC_HAS_BIND)
#define ES_ERR_VO_WBC_NOT_ENABLE ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_WBC_NOT_ENABLE)

/* Channel Relative error code */
#define ES_ERR_VO_CHN_NOT_DISABLE ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_CHN_NOT_DISABLE)
#define ES_ERR_VO_CHN_NOT_ENABLE ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_CHN_NOT_ENABLE)
#define ES_ERR_VO_CHN_NOT_CONFIG ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_CHN_NOT_CONFIG)
#define ES_ERR_VO_CHN_AREA_OVERLAP ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_CHN_AREA_OVERLAP)

/* MISCellaneous error code*/
#define ES_ERR_VO_WAIT_TIMEOUT ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_WAIT_TIMEOUT)
#define ES_ERR_VO_INVALID_VFRAME ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_INVALID_VFRAME)
#define ES_ERR_VO_INVALID_RECT_PARA ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_INVALID_RECT_PARA)
#define ES_ERR_VO_SETBEGIN_ALREADY ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_SETBEGIN_ALREADY)
#define ES_ERR_VO_SETBEGIN_NOTYET ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_SETBEGIN_NOTYET)
#define ES_ERR_VO_SETEND_ALREADY ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_SETEND_ALREADY)
#define ES_ERR_VO_SETEND_NOTYET ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_VO_SETEND_NOTYET)

/* HDMI relative error code */
#define ES_ERR_HDMI_INVALID_ID ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_INVALID_ID)
#define ES_ERR_HDMI_NOT_INIT ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_NOT_INIT)
#define ES_ERR_HDMI_INVALID_PARAM ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_INVALID_PARAM)
#define ES_ERR_HDMI_NULL_PTR ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_NULL_PTR)
#define ES_ERR_HDMI_NOT_CONNECT ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_NOT_CONNECT)
#define ES_ERR_HDMI_READ_SINK_FAILED ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_READ_SINK_FAILED)
#define ES_ERR_HDMI_INIT_ALREADY ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_INIT_ALREADY)
#define ES_ERR_HDMI_CALLBACK_ALREADY ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_CALLBACK_ALREADY)
#define ES_ERR_HDMI_INVALID_CALLBACK ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_INVALID_CALLBACK)
#define ES_ERR_HDMI_BUS_BUSY ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_BUS_BUSY)
#define ES_ERR_HDMI_MALLOC_FAILED ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_MALLOC_FAILED)
#define ES_ERR_HDMI_FEATURE_NO_SUPPORT ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_HDMI_FEATURE_NO_SUPPORT)

/* MIPI relative error code */
#define ES_ERR_MIPI_INVALID_ID ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_MIPI_INVALID_ID)
#define ES_ERR_MIPI_NOT_INIT ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_MIPI_NOT_INIT)
#define ES_ERR_MIPI_INVALID_PARAM ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_MIPI_INVALID_PARAM)
#define ES_ERR_MIPI_NULL_PTR ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_MIPI_NULL_PTR)
#define ES_ERR_MIPI_NOT_CONNECT ES_DEF_ERR(ES_ID_VO, EN_ERR_LEVEL_ERROR, ERR_MIPI_NOT_CONNECT)

/* VO video output interface type */
#define VO_INTF_CVBS (0x01L << 0)
#define VO_INTF_YPBPR (0x01L << 1)
#define VO_INTF_VGA (0x01L << 2)
#define VO_INTF_BT656 (0x01L << 3)
#define VO_INTF_BT1120 (0x01L << 4)
#define VO_INTF_HDMI (0x01L << 5)
#define VO_INTF_LCD (0x01L << 6)
#define VO_INTF_BT656_H (0x01L << 7)
#define VO_INTF_BT656_L (0x01L << 8)
#define VO_INTF_LCD_6BIT (0x01L << 9)
#define VO_INTF_LCD_8BIT (0x01L << 10)
#define VO_INTF_LCD_16BIT (0x01L << 11)
#define VO_INTF_LCD_18BIT (0x01L << 12)
#define VO_INTF_LCD_24BIT (0x01L << 13)
#define VO_INTF_MIPI (0x01L << 14)
#define VO_INTF_MIPI_SLAVE (0x01L << 15)
#define VO_INTF_VIRTUAL (0x01L << 16)

typedef ES_U32 VO_INTF_TYPE_E;

typedef enum esVO_INTF_SYNC_E {
    VO_OUTPUT_PAL = 0, /* PAL standard*/
    VO_OUTPUT_NTSC,    /* NTSC standard */

    VO_OUTPUT_1080P24, /* 1920 x 1080 at 24 Hz. */
    VO_OUTPUT_1080P25, /* 1920 x 1080 at 25 Hz. */
    VO_OUTPUT_1080P30, /* 1920 x 1080 at 30 Hz. */
    VO_OUTPUT_1080P48, /* 1920 x 1080 at 48 Hz. */

    VO_OUTPUT_720P25,   /* 1280 x  720 at 25 Hz. */
    VO_OUTPUT_720P30,   /* 1280 x  720 at 30 Hz. */
    VO_OUTPUT_720P50,   /* 1280 x  720 at 50 Hz. */
    VO_OUTPUT_720P60,   /* 1280 x  720 at 60 Hz. */
    VO_OUTPUT_720P100,  /* 1280 x  720 at 100 Hz. */
    VO_OUTPUT_720P120,  /* 1280 x  720 at 120 Hz. */
    VO_OUTPUT_1080I50,  /* 1920 x 1080 at 50 Hz, interlace. */
    VO_OUTPUT_1080I60,  /* 1920 x 1080 at 60 Hz, interlace. */
    VO_OUTPUT_1080P50,  /* 1920 x 1080 at 50 Hz. */
    VO_OUTPUT_1080P60,  /* 1920 x 1080 at 60 Hz. */
    VO_OUTPUT_1080P100, /* 1920 x 1080 at 100 Hz. */
    VO_OUTPUT_1080P120, /* 1920 x 1080 at 120 Hz. */

    VO_OUTPUT_576P50,  /* 720 x 576 at 50 Hz. */
    VO_OUTPUT_576P100, /* 720 x 576 at 100 Hz */
    VO_OUTPUT_576P200, /* 720 x 576 at 200 Hz */
    VO_OUTPUT_480P60,  /* 720 x 480 at 60 Hz. */
    VO_OUTPUT_480P120, /* 720 x 480 at 120 Hz */
    VO_OUTPUT_480P240, /* 720 x 480 at 240 Hz */

    VO_OUTPUT_800x600_60,   /* VESA 800 x 600 at 60 Hz (non-interlaced) */
    VO_OUTPUT_1024x768_60,  /* VESA 1024 x 768 at 60 Hz (non-interlaced) */
    VO_OUTPUT_1152x864_75,  /* VESA 1152 x 864 at 75 Hz (non-interlaced) */
    VO_OUTPUT_1280x1024_60, /* VESA 1280 x 1024 at 60 Hz (non-interlaced) */
    VO_OUTPUT_1366x768_60,  /* VESA 1366 x 768 at 60 Hz (non-interlaced) */
    VO_OUTPUT_1440x900_60,  /* VESA 1440 x 900 at 60 Hz (non-interlaced) CVT Compliant */
    VO_OUTPUT_1440x480_60,  /* VESA 1440 x 480 at 60 Hz (non-interlaced) */
    VO_OUTPUT_1440x576_50,  /* VESA 1440 x 576 at 50 Hz (non-interlaced) */
    VO_OUTPUT_1280x800_60,  /* 1280*800@60Hz VGA@60Hz*/
    VO_OUTPUT_1600x1200_60, /* VESA 1600 x 1200 at 60 Hz (non-interlaced) */
    VO_OUTPUT_1600x900_60,  /* VESA 1600 x 900 at 60 Hz (non-interlaced) */
    VO_OUTPUT_1680x1050_60, /* VESA 1680 x 1050 at 60 Hz (non-interlaced) */
    VO_OUTPUT_1920x1200_60, /* VESA 1920 x 1600 at 60 Hz (non-interlaced) CVT (Reduced Blanking)*/
    VO_OUTPUT_640x480_60,   /* VESA 640 x 480 at 60 Hz (non-interlaced) CVT */
    VO_OUTPUT_960H_PAL,     /* ITU-R BT.1302 960 x 576 at 50 Hz (interlaced)*/
    VO_OUTPUT_960H_NTSC,    /* ITU-R BT.1302 960 x 480 at 60 Hz (interlaced)*/
    VO_OUTPUT_1920x2160_30, /* 1920x2160_30 */
    VO_OUTPUT_2560x1440_30, /* 2560x1440_30 */
    VO_OUTPUT_2560x1440_60, /* 2560x1440_60 */
    VO_OUTPUT_2560x1600_60, /* 2560x1600_60 */
    VO_OUTPUT_2880x240_60,  /* 2880x240_60 */
    VO_OUTPUT_2880x288_50,  /* 2880x288_50 */
    VO_OUTPUT_2880x480_60,  /* 2880x480_60 */
    VO_OUTPUT_2880x576_50,  /* 2880x576_50 */
    VO_OUTPUT_3840x2160_24, /* 3840x2160_24 */
    VO_OUTPUT_3840x2160_25, /* 3840x2160_25 */
    VO_OUTPUT_3840x2160_30, /* 3840x2160_30 */
    VO_OUTPUT_3840x2160_48, /* 3840x2160_48 */
    VO_OUTPUT_3840x2160_50, /* 3840x2160_50 */
    VO_OUTPUT_3840x2160_60, /* 3840x2160_60 */
    VO_OUTPUT_4096x2160_24, /* 4096x2160_24 */
    VO_OUTPUT_4096x2160_25, /* 4096x2160_25 */
    VO_OUTPUT_4096x2160_30, /* 4096x2160_30 */
    VO_OUTPUT_4096x2160_48, /* 4096x2160_48 */
    VO_OUTPUT_4096x2160_50, /* 4096x2160_50 */
    VO_OUTPUT_4096x2160_60, /* 4096x2160_60 */
    VO_OUTPUT_320x240_60,   /* For ota5182 at 60 Hz (8bit)  */
    VO_OUTPUT_320x240_50,   /* For ili9342 at 50 Hz (6bit)  */
    VO_OUTPUT_240x320_50,   /* 240x320_50 */
    VO_OUTPUT_240x320_60,   /* For ili9341 at 60 Hz (16bit) */
    VO_OUTPUT_800x600_50,   /* For LCD     at 50 Hz (24bit) */
    VO_OUTPUT_720x1280_60,  /* For MIPI DSI Tx 720 x1280 at 60 Hz */
    VO_OUTPUT_1080x1920_60, /* For MIPI DSI Tx 1080x1920 at 60 Hz */
    VO_OUTPUT_7680x4320_30, /* For HDMI2.1 at 30 Hz         */
    VO_OUTPUT_USER,         /* User timing. */

    VO_OUTPUT_BUTT

} VO_INTF_SYNC_E;

typedef enum esVO_ZOOM_IN_E {
    VO_ZOOM_IN_RECT = 0, /* Zoom in by rect */
    VO_ZOOM_IN_RATIO,    /* Zoom in by ratio */
    VO_ZOOM_IN_BUTT
} VO_ZOOM_IN_E;

typedef enum esVO_CSC_MATRIX_E {
    VO_CSC_MATRIX_IDENTITY = 0, /* Identity CSC matrix. */

    VO_CSC_MATRIX_BT601_TO_BT709, /* BT601 to BT709 */
    VO_CSC_MATRIX_BT709_TO_BT601, /* BT709 to BT601 */

    VO_CSC_MATRIX_BT601_TO_RGB_PC, /* BT601 to RGB */
    VO_CSC_MATRIX_BT709_TO_RGB_PC, /* BT709 to RGB */

    VO_CSC_MATRIX_RGB_TO_BT601_PC, /* RGB to BT601 FULL */
    VO_CSC_MATRIX_RGB_TO_BT709_PC, /* RGB to BT709 FULL */

    VO_CSC_MATRIX_RGB_TO_BT2020_PC, /* RGB to BT.2020 */
    VO_CSC_MATRIX_BT2020_TO_RGB_PC, /* BT.2020 to RGB */

    VO_CSC_MATRIX_RGB_TO_BT601_TV, /* RGB to BT601 LIMIT */
    VO_CSC_MATRIX_RGB_TO_BT709_TV, /* RGB to BT709 LIMIT */

    VO_CSC_MATRIX_BUTT
} VO_CSC_MATRIX_E;

typedef enum esVO_DISPLAY_BUF_LEN_E {
    VO_MIN_BUF_LEN = 0,
    VO_DEF_BUF_LEN = 3,
    VO_MAX_BUF_LEN = 15,
    VO_BUF_LEN_BUTT,
} VO_DISPLAY_BUF_LEN_E;

typedef struct esVO_CHN_ATTR_S {
    ES_U32 priority;    /* Video out overlay pri sd */
    RECT_S rect;        /* Rectangle of video output channel */
    ES_BOOL bDeflicker; /* Deflicker or not sd */
} VO_CHN_ATTR_S;

typedef struct esVO_CHN_PARAM_S {
    ASPECT_RATIO_S aspectRatio; /* RW; aspect ratio */
} VO_CHN_PARAM_S;

typedef struct esVO_BORDER_S {
    ES_BOOL bBorderEn; /* RW; Do frame or not*/
    BORDER_S border;   /* RW; frame's top, bottom, left, right width and color*/
} VO_BORDER_S;

typedef struct esVO_QUERY_STATUS_S {
    ES_U32 chnBufUsed; /* Channel buffer that been occupied */
} VO_QUERY_STATUS_S;

typedef struct esVO_SYNC_INFO_S {
    ES_BOOL bSynm; /* RW; sync mode(0:timing,as BT.656; 1:signal,as LCD) */
    ES_BOOL bIop;  /* RW; interlaced or progressive display(0:i; 1:p) */
    ES_U8 intfb;   /* RW; interlace bit width while output */

    ES_U16 vact; /* RW; vertical active area */
    ES_U16 vbb;  /* RW; vertical back blank porch */
    ES_U16 vfb;  /* RW; vertical front blank porch */

    ES_U16 hact; /* RW; horizontal active area */
    ES_U16 hbb;  /* RW; horizontal back blank porch */
    ES_U16 hfb;  /* RW; horizontal front blank porch */
    ES_U16 hmid; /* RW; bottom horizontal active area */

    ES_U16 bvact; /* RW; bottom vertical active area */
    ES_U16 bvbb;  /* RW; bottom vertical back blank porch */
    ES_U16 bvfb;  /* RW; bottom vertical front blank porch */

    ES_U16 hpw; /* RW; horizontal pulse width */
    ES_U16 vpw; /* RW; vertical pulse width */

    ES_BOOL bIdv; /* RW; inverse data valid of output */
    ES_BOOL bIhs; /* RW; inverse horizontal synch signal */
    ES_BOOL bIvs; /* RW; inverse vertical synch signal */

} VO_SYNC_INFO_S;

typedef struct esVO_PUB_ATTR_S {
    ES_U32 bgColor;          /* RW; Background color of a device, in RGB format. */
    VO_INTF_TYPE_E intfType; /* RW; Type of a VO interface */
    VO_INTF_SYNC_E intfSync; /* RW; Type of a VO interface timing */
    VO_SYNC_INFO_S syncInfo; /* RW; Information about VO interface timings */
} VO_PUB_ATTR_S;

typedef enum esVO_PART_MODE_E {
    VO_PART_MODE_SINGLE = 0, /* single partition, which use software to make multi-picture in one hardware cell */
    VO_PART_MODE_MULTI = 1,  /* muliti partition, each partition is a hardware cell */
    VO_PART_MODE_BUTT

} VO_PART_MODE_E;

typedef struct esVO_VIDEO_LAYER_ATTR_S {
    RECT_S dispRect;                 /* RW; Display resolution */
    SIZE_S imageSize;                /* RW; Canvas size of the video layer */
    ES_U32 dispFrmRt;                /* RW; Display frame rate */
    PIXEL_FORMAT_E pixFormat;        /* RW; Pixel format of the video layer */
    ES_BOOL bDoubleFrame;            /* RW; Whether to double frames */
    ES_BOOL bClusterMode;            /* RW; Whether to take Cluster way to use memory*/
    DYNAMIC_RANGE_E dstDynamicRange; /* RW; Video Layer output dynamic range type*/
} VO_VIDEO_LAYER_ATTR_S;

typedef struct esVO_LAYER_PARAM_S {
    ASPECT_RATIO_S aspectRatio; /* RW; aspect ratio */
} VO_LAYER_PARAM_S;

typedef struct esVO_ZOOM_RATIO_S {
    ES_U32 xRatio; /* RW; Range: [0,1000]; XRatio = x * 1000 / W , x means the start point to be zoomed, W means
                          displaying channel's width. */
    ES_U32 yRatio; /* RW; Range: [0,1000]; YRatio = y * 1000 / H , y means the start point to be zoomed, H means
                          displaying channel's height. */
    ES_U32 wRatio; /* RW; Range: [0,1000]; WRatio = w * 1000 / W , w means the width to be zoomed, W means
                          displaying channel's width. */
    ES_U32 hRatio; /* RW; Range: [0,1000]; HRatio = h * 1000 / H , h means the height to be zoomed, H means
                          displaying channel's height. */
} VO_ZOOM_RATIO_S;

typedef struct esVO_ZOOM_ATTR_S {
    VO_ZOOM_IN_E zoomType; /* choose the type of zoom in */
    union {
        RECT_S zoomRect;           /* zoom in by rect */
        VO_ZOOM_RATIO_S zoomRatio; /* zoom in by ratio */
    };
} VO_ZOOM_ATTR_S;

typedef struct esVO_CSC_S {
    VO_CSC_MATRIX_E cscMatrix; /* CSC matrix */
    ES_U32 luma;               /* RW; Range: [0, 100]; luminance, default: 50 */
    ES_U32 contrast;           /* RW; Range: [0, 100]; contrast, default: 50 */
    ES_U32 hue;                /* RW; Range: [0, 100]; hue, default: 50 */
    ES_U32 satuature;          /* RW; Range: [0, 100]; satuature, default: 50 */
} VO_CSC_S;

#define ES_VO_MIN_CHN_WIDTH 32    /* channel minimal width */
#define ES_VO_MIN_CHN_HEIGHT 32   /* channel minimal height */
#define ES_VO_MAX_CHN_WIDTH 4096  /* channel max width */
#define ES_VO_MAX_CHN_HEIGHT 2160 /* channel max height */
#define ES_VO_MAX_ZOOM_RATIO 1000 /* max zoom ratio, 1000 means 100% scale */

/* VO device id */
#define ES_VO_PHY_DEV_0 0  /* physical device */
#define ES_VO_PHY_DEV_1 1  /* physical device */
#define ES_VO_VIRT_DEV_0 2 /* Using hardware simulation */
#define ES_VO_VIRT_DEV_1 3 /* Using software simulation */

/* VO layer id */
#define ES_VO_PHY_LAYER_0 0
#define ES_VO_PHY_LAYER_1 1
#define ES_VO_VIRT_LAYER_0 2
#define ES_VO_VIRT_LAYER_1 3

// Any integer between [-64, 64], <0: play rewind.
#define ES_VO_MAX_CHN_DISP_RATE 64
#define ES_VO_MIN_CHN_DISP_RATE -64

/* play toleration range [1, 100000] */
#define ES_VO_MIN_PLAY_TOLERATION 1
#define ES_VO_MAX_PLAY_TOLERATION 100000

/* dev frame rate range [0, 240] */
#define ES_VO_MIN_FRAME_RATE 0
#define ES_VO_MAX_FRAME_RATE 240

/* vo mode max num*/
#define ES_VO_MODE_MAX 128

typedef enum esHDMI_PIC_ASPECT_RATIO_E {
    ES_HDMI_PIC_ASP_RATIO_NO_DATA, /**<Aspect Ratio unknown*/
    ES_HDMI_PIC_ASP_RATIO_4TO3,    /**<Aspect Ratio 4:3*/
    ES_HDMI_PIC_ASP_RATIO_16TO9,   /**<Aspect Ratio 16:9*/
    ES_HDMI_PIC_ASP_RATIO_64TO27,
    ES_HDMI_PIC_ASP_RATIO_256TO135,
    ES_HDMI_PIC_ASP_RATIO_RESERVED,
} ES_HDMI_PIC_ASPECT_RATIO_E;

typedef struct esVO_DISPLAY_MODE_S {
    ES_U16 modesNum;
    struct {
        ES_U32 clock;
        ES_U16 hdisplay, hsyncStart, hsyncEnd, htotal;
        ES_U16 vdisplay, vsyncStart, vsyncEnd, vtotal;
        ES_U16 refresh;
        ES_U32 type;
        ES_U32 flags;
    } modes[ES_VO_MODE_MAX];
} ES_HDMI_DISPLAY_MODE_S;

typedef enum esHDMI_VIDEO_MODE {
    ES_HDMI_VIDEO_MODE_RGB444 = 0,
    ES_HDMI_VIDEO_MODE_YCBCR444,
    ES_HDMI_VIDEO_MODE_YCBCR422,
    ES_HDMI_VIDEO_MODE_YCBCR420,
    ES_HDMI_VIDEO_MODE_HIGH_SUBSAMPLING,
    ES_HDMI_VIDEO_MODE_LOW_SUBSAMPLING,
    ES_HDMI_VIDEO_MODE_BUTT
} ES_HDMI_VIDEO_MODE_E;

typedef enum esHDMI_DEEP_COLOR_E {
    ES_HDMI_DEEP_COLOR_AUTOMATIC = 0x00,
    ES_HDMI_DEEP_COLOR_24BIT = 8,
    ES_HDMI_DEEP_COLOR_30BIT = 10,
    ES_HDMI_DEEP_COLOR_BUTT
} ES_HDMI_DEEP_COLOR_E;

typedef struct esHDIMI_ATTR_S {
    ES_BOOL enableVideo; /* Whether to output video
                          * HI_TRUE: output normal pictures
                          * HI_FALSE: output blank screens
                          */
    VO_INTF_SYNC_E intfSync;
    ES_HDMI_VIDEO_MODE_E vidOutMode;
    ES_HDMI_DEEP_COLOR_E deepColorMode;
    ES_HDMI_PIC_ASPECT_RATIO_E aspectRatio;
} ES_HDMI_ATTR_S;

#define ES_VO_PRIMARY_NUM 3

typedef enum esHDMI_METADATA_E { HDMI_STATIC_METADATA_TYPE1 = 0, HDMI_STATIC_METADATA_TYPE1_BUTT } ES_HDMI_METADATA_E;

typedef struct esCOORDINATE_S {
    ES_U16 x;
    ES_U16 y;
} COORDINATE_S;

typedef struct esHDMI_HDR_METADATA_S {
    COORDINATE_S displayPrimaries[ES_VO_PRIMARY_NUM]; /*Color Primaries of the Data.The xy coordinates of primaries in
                                                       * the CIE 1931 color space. the index 0 contains red, 1 is for
                                                       * green and 2 is for blue.*/
    COORDINATE_S whitePoint; /*White Point of Colorspace Data.The xy coordinates of white point in the CIE 1931 color
                              * space. each value is normalized to 50000 (meaning that in unit of 0.00002)*/
    ES_U16 maxDisplayLuma;   /*Max Mastering Display Luminance.*/
    ES_U16 minDisplayLuma;   /*Min Mastering Display Luminance.*/
    ES_U16 maxCll;           /*Max Content Light Level.*/
    ES_U16 maxFall;          /*Max Frame Average Light Level.*/
} ES_HDMI_HDR_METADATA_S;

typedef struct esHDIMI_HDR_PARAM_S {
    ES_HDMI_METADATA_E metadataType;
    union {
        ES_HDMI_HDR_METADATA_S hdrMetadata;
    };
} ES_HDIMI_HDR_PARAM_S;

typedef enum esHDMI_ID_E { ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT } ES_HDMI_ID_E;

/**HDMI event type*/
typedef enum ES_HDMI_EVENT_TYPE_E {
    ES_HDMI_EVENT_HOTPLUG = 0x10, /**<HDMI HotPlug event type*/
    ES_HDMI_EVENT_NO_PLUG,        /**<HDMI unplug event type*/
    ES_HDMI_EVENT_BUTT
} ES_HDMI_EVENT_TYPE_E;

/**The hotplug callback function interface */
typedef ES_VOID (*ES_HDMI_EVENT_CALLBACK)(ES_HDMI_EVENT_TYPE_E event, ES_VOID* pPrivateData);

/**HDMI Callback Struct*/
typedef struct ES_HDMI_CALLBACK_FUNC_S {
    ES_HDMI_EVENT_CALLBACK pfnHdmiEventCallback; /**<callback function pointer*/
    ES_VOID* pPrivateData;                       /**<callback funtion param*/
} ES_HDMI_CALLBACK_FUNC_S;

typedef struct esHDMI_EDID_S {
    ES_BOOL edidValid;
    ES_U32 edidlength;
    ES_U8 edid[512];
} ES_HDMI_EDID_S;

/**************************************************************************************************/
#define LANE_MAX_NUM 4

typedef enum esMIPI_ID_E { ES_MIPI_ID_0 = 0, ES_MIPI_ID_BUTT } ES_MIPI_ID_E;

typedef enum MIPI_OUT_FORMAT_E {
    MIPI_OUT_FORMAT_RGB_16_BIT = 0x0,
    MIPI_OUT_FORMAT_RGB_18_BIT,
    MIPI_OUT_FORMAT_RGB_24_BIT,
    MIPI_OUT_FORMAT_BUTT
} MIPI_OUT_FORMAT_E;

typedef enum MIPI_OUT_MODE_E {
    MIPI_OUTPUT_MODE_CSI = 0x0,
    MIPI_OUTPUT_MODE_DSI_VIDEO,
    MIPI_OUTPUT_MODE_DSI_CMD,
    MIPI_OUTPUT_MODE_DSI_BUTT
} MIPI_OUT_MODE_E;

typedef struct MIPI_DEV_ATTR {
    MIPI_OUT_FORMAT_E outputFormat;
    MIPI_OUT_MODE_E outputMode;
    ES_U32 lanes;
} MIPI_DEV_ATTR_S;

typedef struct ES_MIPI_CMD_S {
    ES_U16 dataType;
    ES_U16 cmdSize;
    ES_U32 sleepUs;
    ES_U8* pCmd;
} ES_MIPI_CMD_S;

/***************************************************************************************************/

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* End of #ifdef __cplusplus */

#endif /* End of #ifndef __ES_COMM_VO_H__ */
