/*
 * Copyright © 2023 ESWIN. All rights reserved.
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
 * all intellectual property and proprietary rights in and to this software.Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed,reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 * @Description: Define all the data stuctures.
 * @Author: ESWIN
 */

#ifndef __ES_COMM_VPS_H__
#define __ES_COMM_VPS_H__

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

#include "es_type.h"
#include "es_common.h"
#include "es_errno.h"
#include "es_comm_video.h"

#define VPS_SHARPEN_GAIN_NUM (32)
#define VPS_AUTO_ISO_STRENGTH_NUM (16)
#define VPS_YUV_SHPLUMA_NUM (32)
#define VPS_3DNR_MAX_AUTO_PARAM_NUM (16)

#define ES_ERR_VPS_NULL_PTR ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_NULL_PTR)
#define ES_ERR_VPS_NOTREADY ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_SYS_NOTREADY)
#define ES_ERR_VPS_INVALID_GRPID ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_DEVID)
#define ES_ERR_VPS_INVALID_CHNID ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_CHNID)
#define ES_ERR_VPS_EXIST ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_EXIST)
#define ES_ERR_VPS_UNEXIST ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_UNEXIST)
#define ES_ERR_VPS_NOT_SUPPORT ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_SUPPORT)
#define ES_ERR_VPS_NOT_PERM ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_PERM)
#define ES_ERR_VPS_NOMEM ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_NOMEM)
#define ES_ERR_VPS_NOBUF ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_NOBUF)
#define ES_ERR_VPS_ILLEGAL_PARAM ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_ILLEGAL_PARAM)
#define ES_ERR_VPS_BUSY ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_BUSY)
#define ES_ERR_VPS_BUF_EMPTY ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_EMPTY)
#define ES_ERR_VPS_BUF_FULL ES_DEF_ERR(ES_ID_VPS, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_FULL)

#define VPS_INVALID_FRMRATE (-1)
#define VPS_CHN0 (0)
#define VPS_CHN1 (1)
#define VPS_CHN2 (2)
#define VPS_INVALID_CHN (-1)
#define VPS_OVERLAY_MAX_DISPLAY_NUM (4)
#define VPS_OVERLAY_MAX_LINE_NUM (8)
#define VPS_OVERLAY_MAX_QUADRANGLE_NUM (8)

#define VPS_MIN_LINE_WIDTH (2)
#define VPS_MAX_LINE_WIDTH (16)

#define VPS_MIN_STRIDE_ALIGN (64)
#define VPS_MIN_HEIGHT_ALIGN (8)
#define VPS_CSC_TABLE_SIZE (12)

typedef ES_VOID* VPS_OVERLAY_HANDLE;

typedef struct esVPS_GRP_ATTR_S {
    ES_U32 maxW;                  /* RW; Width of source image. */
    ES_U32 maxH;                  /* RW; Height of source image. */
    PIXEL_FORMAT_E pixelFormat;   /* RW; Pixel format of source image. */
    DYNAMIC_RANGE_E dynamicRange; /* RW; Dynamic range of source image. */
    FRAME_RATE_CTRL_S frameRate;  /* Grp frame rate control. */
} VPS_GRP_ATTR_S;

typedef enum esHWTYPE_E {
    HW_TYPE_HAE,
    HW_TYPE_3D,
    HW_TYPE_DWE,
    HW_TYPE_VSE,
    HW_TYPE_BUTT,
} HWTYPE_E;

typedef struct esVPS_CHN_ATTR_S {
    ES_U32 width;                 /* RW; Width of target image. */
    ES_U32 height;                /* RW; Height of target image. */
    VIDEO_FORMAT_E videoFormat;   /* RW; Video format of target image. */
    PIXEL_FORMAT_E pixelFormat;   /* RW; Pixel format of target image. */
    DYNAMIC_RANGE_E dynamicRange; /* RW; DynamicRange of target image. */
    COMPRESS_MODE_E compressMode; /* RW; Compression mode of the output. */
    FRAME_RATE_CTRL_S frameRate;  /* Frame rate control info */
    ES_BOOL bMirror;              /* RW; Mirror enable. */
    ES_BOOL bFlip;                /* RW; Flip enable. */
    ES_U32 depth;                 /* RW; Range: [0, 8]; User get list depth. */
    ASPECT_RATIO_S aspectRatio;   /* Aspect Ratio info. */
    HWTYPE_E resizeHwType;        /* RW; Appoint the hardware to execute resize work. */
    HWTYPE_E mirrorFlipHwType;    /* RW; Appoint the hardware to execute mirror and/or flip work. */
    HWTYPE_E aspectHwType;        /* RW; Appoint the hardware to execute aspect work. */
} VPS_CHN_ATTR_S;

typedef enum esVPS_CROP_COORDINATE_E {
    VPS_CROP_RATIO_COOR = 0, /* Ratio coordinate. */
    VPS_CROP_ABS_COOR        /* Absolute coordinate. */
} VPS_CROP_COORDINATE_E;

typedef struct esVPS_CROP_INFO_S {
    ES_BOOL bEnable;                      /* RW; Range: [0, 1]; CROP enable. */
    VPS_CROP_COORDINATE_E cropCoordinate; /* RW;  Range: [0, 1]; Coordinate mode of the crop start point. */
    RECT_S cropRect;                      /* CROP rectangular. */
} VPS_CROP_INFO_S;

typedef struct esVPS_LDC_ATTR_S {
    ES_BOOL bEnable; /* RW; Range: [0, 1]; Whether LDC is enbale */
    LDC_ATTR_S attr;
} VPS_LDC_ATTR_S;

typedef struct esVPS_PARAM_MOD_S {
    ES_U32 vpsVbSource; /* 0: Common VB pool, 2: User VB pool. */
} VPS_MOD_PARAM_S;

typedef enum _VPS_BLEND_MODE {
    VPS_BLEND_CLEAR = 0,    /* 0         0           */
    VPS_BLEND_SRC,          /* 1         0           */
    VPS_BLEND_DST,          /* 0         1           */
    VPS_BLEND_SRC_OVER_DST, /* 1         1 - Asrc    */
    VPS_BLEND_DST_OVER_SRC, /* 1 - Adst  1           */
    VPS_BLEND_SRC_IN_DST,   /* Adst      0           */
    VPS_BLEND_DST_IN_SRC,   /* 0         Asrc        */
    VPS_BLEND_SRC_OUT_DST,  /* 1 - Adst  0           */
    VPS_BLEND_DST_OUT_SRC,  /* 0         1 - Asrc    */
    VPS_BLEND_SRC_ATOP_DST, /* Adst      1 - Asrc    */
    VPS_BLEND_DST_ATOP_SRC, /* 1 - Adst  Asrc        */
    VPS_BLEND_ADD,          /* 1         1           */
    VPS_BLEND_SRC_XOR_DST,  /* 1 - Adst  1 - Asrc    */

    VPS_BLEND_BUTT,
} VPS_BLEND_MODE_E;

typedef enum _VPS_NORMALIZATION_MODE {
    VPS_NORMALIZATION_MIN_MAX,  // min-max
    VPS_NORMALIZATION_Z_SCORE
} VPS_NORMALIZATION_MODE_E;

/**
 * float32 format to 4bytes,
 * normalization would given those values
 */
typedef struct _VPS_RGB {
    ES_U32 r;
    ES_U32 g;
    ES_U32 b;
} VPS_RGB_S;

typedef struct _VPS_NORMALIZATION_PARAMS {
    VPS_NORMALIZATION_MODE_E normalizationMode;
    VPS_RGB_S minValue;
    VPS_RGB_S maxMinReciprocal;
    VPS_RGB_S stdReciprocal;
    VPS_RGB_S meanValue;
    ES_U32 stepReciprocal;
    ES_BOOL bByPassQuantization;
} VPS_NORMALIZATION_PARAMS_S;

typedef struct esVPS_NORMALIZATION_INFO_S {
    ES_BOOL bEnable;
    VPS_NORMALIZATION_PARAMS_S param;
} VPS_NORMALIZATION_INFO_S;

typedef enum _VPS_DEWARP_MODEL_E {
    VPS_DEWARP_MODEL_LENS_DISTORTION_CORRECTION = 0,
    VPS_DEWARP_MODEL_FISHEYE_EXPAND,
    VPS_DEWARP_MODEL_SPLIT_SCREEN,
    VPS_DEWARP_MODEL_FISHEYE_DEWARP,
    VPS_DEWARP_MODEL_PERSPECTIVE,
    VPS_DEWARP_MODEL_BYPASS,
    VPS_DEWARP_MODEL_BUTT,
} VPS_DEWARP_MODEL_E;

typedef struct esVPS_DEWARP_DISTORTION_MAP_S {
    ES_U32 userMapSize; /* 0: set camera matrix, calculate map at driver.
                         * !0: set distortion map, calculated by user. */
    ES_DOUBLE cameraMatrix[9];
    ES_DOUBLE perspectiveMatrix[9];
    ES_DOUBLE distortionCoeff[8];
    ES_U32* pUserMap;
} VPS_DEWARP_DISTORTION_MAP_S;

typedef struct esVPS_DEWARP_FOV_SETTING_S {
    ES_DOUBLE offAngleUL;
    ES_DOUBLE offAngleUR;
    ES_DOUBLE offAngleDL;
    ES_DOUBLE offAngleDR;
    ES_DOUBLE fovUL;
    ES_DOUBLE fovUR;
    ES_DOUBLE fovDL;
    ES_DOUBLE fovDR;
    ES_S32 panoAtWin;
    ES_DOUBLE centerOffsetRatioUL;
    ES_DOUBLE centerOffsetRatioUR;
    ES_DOUBLE centerOffsetRatioDL;
    ES_DOUBLE centerOffsetRatioDR;
    ES_DOUBLE circleOffsetRatioUL;
    ES_DOUBLE circleOffsetRatioUR;
    ES_DOUBLE circleOffsetRatioDL;
    ES_DOUBLE circleOffsetRatioDR;
} VPS_DEWARP_FOV_SETTING_S;

typedef struct esVPS_DEWARP_PARAMS_S {
    ES_BOOL bEnable;
    VPS_DEWARP_MODEL_E dewarpModel;
    VPS_DEWARP_DISTORTION_MAP_S map;
    VPS_DEWARP_FOV_SETTING_S fov;
    ES_DOUBLE scaleFactor; /* Up scale factor[1.0, 4.0] */
    ES_U32 splitHorizonLine;
    ES_U32 splitVerticalLineUp;
    ES_U32 splitVerticalLineDown;
    ES_U32 roiX;
    ES_U32 roiY;
} VPS_DEWARP_PARAMS_S;

typedef struct esVPS_OUT_ATTR_S {
    ES_BOOL bEnable;
    PIXEL_FORMAT_E pixelFormat;
    ES_U32 width;
    ES_U32 height;
} VPS_OUT_ATTR_S;

typedef struct esVPS_MULTI_OUT_ATTR_S {
    VPS_OUT_ATTR_S outAttr[ES_VPS_MAX_CHN_NUM];
} VPS_MULTI_OUT_ATTR_S;

typedef struct esVPS_CHN_FRAME_STATUS_S {
    ES_U32 chnDropFrmCnt;      // The count of frame channel processed success but dropped directly.
    ES_U32 chnOutputFrmCnt;    // The count of frame channel processed success and cached in output FIFO.
    ES_U32 chnFailFrmCnt;      // The count of frame channel processed failed.
    ES_U32 chnConsumedFrmCnt;  // The count of frame consumed from this channel FIFO.
} VPS_CHN_FRAME_STATUS_S;

/**
 * grpRecvFrmCnt = grpRejectFrmCnt + grpAcceptFrmCnt + grpDropFrmCnt + grpFailFrmCnt;
 * if channel enable, grpAcceptFrmCnt = chnDropFrmCnt + chnOutputFrmCnt + chnFailFrmCnt;
 */
typedef struct esVPS_QUERY_STATUS_S {
    ES_U32 grpRecvFrmCnt;    // The count of frame VPS group received.
    ES_U32 grpRejectFrmCnt;  // The count of frame VPS group rejected by input FIFO full.
    ES_U32 grpAcceptFrmCnt;  // The count of frame VPS group accepted, cached in input FIFO.
    ES_U32 grpDropFrmCnt;    // The count of frame VPS group accepted, but not processed and dropped directly.
    ES_U32 grpFailFrmCnt;    // The count of frame VPS group accepted failed.
    VPS_CHN_FRAME_STATUS_S chnFrmStatus[ES_VPS_MAX_CHN_NUM];
} VPS_QUERY_STATUS_S;

typedef enum esVPS_OVERLAY_TYPE_E {
    VPS_OVERLAY_TYPE_LINE = 0,
    VPS_OVERLAY_TYPE_RECT = 1,
    VPS_OVERLAY_TYPE_POLYGON = 2, /* Not support currently */
    VPS_OVERLAY_TYPE_MOSAIC = 3,
    VPS_OVERLAY_TYPE_BITMAP = 4,
    VPS_OVERLAY_TYPE_BUTT
} VPS_OVERLAY_TYPE_E;

typedef enum esVPS_MOSAIC_BLK_SIZE_E {
    VPS_MOSAIC_BLK_SIZE_8 = 0, /* block size 8*8 of MOSAIC */
    VPS_MOSAIC_BLK_SIZE_16,    /* block size 16*16 of MOSAIC */
    VPS_MOSAIC_BLK_SIZE_32,    /* block size 32*32 of MOSAIC */
    VPS_MOSAIC_BLK_SIZE_64,    /* block size 64*64 of MOSAIC */
    VPS_MOSAIC_BLK_SIZE_BUTT
} VPS_MOSAIC_BLK_SIZE_E;

typedef struct esVPS_OVERLAY_DISP_LINE_S {
    POINT_S startPoint; /* Start point of line. When use HAE, x and y must multiple of 2 */
    POINT_S endPoint;   /* End point of line. When use HAE, x and y must multiple of 2 */
    ES_U32 lineWidth;   /* [VPS_MIN_LINE_WIDTH, VPS_MAX_LINE_WIDTH]. When use HAE, must multiple of 2 */
    ES_U32 color;       /* ARGB color */
} VPS_OVERLAY_DISP_LINE_S;

typedef struct esVPS_OVERLAY_DISP_LINE_GRP_S {
    ES_U8 lineCnt; /* The valid count of lines, [0, VPS_OVERLAY_MAX_LINE_NUM] */
    VPS_OVERLAY_DISP_LINE_S line[VPS_OVERLAY_MAX_LINE_NUM];
} VPS_OVERLAY_DISP_LINE_GRP_S;

typedef struct esVPS_OVERLAY_DISP_QUADRANGLE_S {
    union {
        RECT_S rect; /* Used when type is VPS_OVERLAY_TYPE_RECT. When use HAE, x, y, width, height must multiple of 2 */
        POINT_S points[4]; /* Used when type is VPS_OVERLAY_TYPE_POLYGON, clockwise */
    } u;
    ES_U32 lineWidth; /* [VPS_MIN_LINE_WIDTH, VPS_MAX_LINE_WIDTH]. When use HAE, must multiple of 2 */
    ES_U32 color;     /* ARGB color */
} VPS_OVERLAY_DISP_QUADRANGLE_S;

typedef struct esVPS_OVERLAY_DISP_QUADRANGLE_GRP_S {
    ES_U8 quadrangleCnt; /* The valid count of quadrangles, [0, VPS_OVERLAY_MAX_QUADRANGLE_NUM] */
    VPS_OVERLAY_DISP_QUADRANGLE_S quadrangle[VPS_OVERLAY_MAX_QUADRANGLE_NUM];
} VPS_OVERLAY_DISP_QUADRANGLE_GRP_S;

typedef struct esVPS_OVERLAY_DISP_MOSAIC_S {
    RECT_S rect;
    VPS_MOSAIC_BLK_SIZE_E blkSize;
} VPS_OVERLAY_DISP_MOSAIC_S;

typedef struct esVPS_OVERLAY_DISP_BITMAP_S {
    PIXEL_FORMAT_E format; /* pixel format of bitmap */
    ES_S32 alpha;          /* global alpha for bitmap[-1, 255], -1: disable, 0: transparent, 255: opaque */
    ES_U16 x;              /* postion x in pixel, must align to 16 pixel */
    ES_U16 y;              /* postion y in pixel, must be even number */
    ES_U16 w;              /* bmp width in pixel, must align to 16 pixel */
    ES_U16 h;              /* bmp height in pixel, must be even number */
    ES_U64 fd;             /* the fd of bmp buffer */
    ES_U32 size;           /* bmp image size(byte) */
    ES_U32 stride;         /* stride must be 256 align */
} VPS_OVERLAY_DISP_BITMAP_S;

typedef struct esVPS_OVERLAY_DISPLAY_S {
    VPS_OVERLAY_TYPE_E type; /* Indicate the type of current overlay display */
    HWTYPE_E hwType;         /* Indicate the hardware type of current overlay display, not effected after grp started */
    union {
        VPS_OVERLAY_DISP_LINE_GRP_S lineGrp;
        VPS_OVERLAY_DISP_QUADRANGLE_GRP_S quadrangleGrp;
        VPS_OVERLAY_DISP_MOSAIC_S mosaic;
        VPS_OVERLAY_DISP_BITMAP_S bitmap;
    } display; /* The attribute of current overlay display */
} VPS_OVERLAY_DISPLAY_S;

typedef struct esVPS_OVERLAY_GROUP_S {
    ES_U32 dispCnt; /* The valid count of displays, [0, VPS_OVERLAY_MAX_DISPLAY_NUM] */
    VPS_OVERLAY_DISPLAY_S disp[VPS_OVERLAY_MAX_DISPLAY_NUM];
} VPS_OVERLAY_GROUP_S;

typedef enum esVPS_RESIZE_METHOD_E {
    /** Bresenham algorithm is used to generate corresponding coordinates for fast stretching,
     * fastest speed with lower quality. This is default value
     */
    VPS_RESIZE_STRETCHBLIT,
    /** using an FIR re-sampling filter, block size is 5x5.
     * slower speed with high quality scaling up or down
     */
    VPS_RESIZE_FILTERBLIT,
    /* using Bilinear algorithm */
    VPS_RESIZE_BILINEAR,
    /* using Bicubic algorithm */
    VPS_RESIZE_BICUBIC,
    VPS_RESIZE_BUTT,
} VPS_RESIZE_METHOD_E;

/**
 * vps color mode,
 * BT601,BT709,BT2020 Y,U,V is limited.
 * for 8bits, limit Y clamp(16,235), U clamp to (16,240), V clamp to (16,240)
 * for 10bits, limit Y clamp(64,940), U clamp to (64,960), V clamp to (64,960)
 * without color range limit, you should use VPS_COLOR_MODE_USER_DEFINED.
 */
typedef enum esVPS_YUV_COLOR_MODE_E {
    VPS_COLOR_MODE_BT601,
    VPS_COLOR_MODE_BT709, /* default value */
    VPS_COLOR_MODE_BT2020,
    VPS_COLOR_MODE_BT601_UNLIMITED,
    VPS_COLOR_MODE_BT709_UNLIMITED,
    VPS_COLOR_MODE_BT2020_UNLIMITED,
    VPS_COLOR_MODE_USER_DEFINED,
    VPS_COLOR_MODE_USER_DEFINED_CLAMP, /* Clamped User defined CSC coefficients */
} VPS_YUV_COLOR_MODE_E;

typedef enum esVPS_HAE_MULTICORE_MODE_E {
    /* hae has 2 hw cores, default it work as combined mode. for small image it maybe slower */
    VPS_HAE_MULTICORE_COMBINED_MODE = 0,
    VPS_HAE_MULTICORE_INDEPENDENT_MODE_CORE0,
    VPS_HAE_MULTICORE_INDEPENDENT_MODE_CORE1,
} VPS_HAE_MULTICORE_MODE_E;

typedef enum esVPS_PROPERTY_TYPE_E {
    /* set/get scaling up or down algorithm */
    VPS_PROPERTY_RESIZE_METHOD,
    /**
     * csc table between yuv and rgb. As hw is 10bit, this table should be expand to 10bits
     * those csc table only affect color mode with VPS_MODE_USER_DEFINED or VPS_MODE_USER_DEFINED_CLAMP
     * yuv to rgb table
     * R = ((C00*Y + C01*U + C02*V) + D0)>>10
     * G = ((C10*Y + C11*U + C12*V) + D1)>>10
     * B = ((C20*Y + C21*U + C22*V) + D2)>>10
     * The array contains the coefficients in the sequence:
     * C00, C01, C02, C10, C11, C12, C20, C21, C22, D0, D1, D2.
     * for example,
     *   yuv2rgb[] = {
             1196, 0, 1724,
             1196,-192, -668,
             1196, 2200,0,
             -958720, 364288, -1202432
         };
      D0=1196*(-64)              + 1724 *(-512)+512 = -958720
      D1=1196*(-64)+(-192)*(-512)+(-668)*(-512)+512 = 364288
      D2=1196*(-64)+ 2200*(-512)               +512 = -1202432

      for BT601 calculation:(for 8bit)
      R‌ = 1.164(Y-16) + 1.596(Cr-128)
‌      G‌ = 1.164(Y-16) - 0.391(Cb-128) - 0.813(Cr-128)
‌      B‌ = 1.164(Y-16) + 2.018(Cb-128)

      fixed-point to speed calculate,(for 10bit)
      R‌ = 1.164(Y-64) + 1.596(Cr-512)
        = (1.164(Y-64) + 1.596(Cr-512))*1024/1024
        = 1192*Y+           1634*V + (-913047)
      G‌ = 1.164(Y-64) - 0.391(Cb-512) - 0.813(Cr-512)
        = (1.164(Y-64) - 0.391(Cb-512) - 0.813(Cr-512))*1024/1024
        = 1192*Y+(-400)*U+(-833)*V + 554958
      B = 1.164(Y-64) + 2.018(Cb-512)
        = (1.164(Y-64) + 2.018(Cb-512))*1024/1024
        = 1192*Y+  2066*U+         + (-1134297)
      so the table, {1192,0,1634,1192,-400,-833,1192,2066,0,-913047,554958,-1134297}

     */
    VPS_PROPERTY_ARRAY_CSC_YUV_TO_RGB,
    /**
     * csc table between yuv and rgb. As hw is 10bit, this table should be expand to 10bits
     * those csc table only affect color mode with VPS_MODE_USER_DEFINED or VPS_MODE_USER_DEFINED_CLAMP
     * rgb to yuv table
     * Y = ((C00*R + C01*G + C02*B) + 512 + D0)>>10
     * U = ((C10*R + C11*G + C12*B) + 512 + D1)>>10
     * V = ((C20*R + C21*G + C22*B) + 512 + D2)>>10
     * The array contains the coefficients in the sequence:
     * C00, C01, C02, C10, C11, C12, C20, C21, C22, D0, D1, D2.
     * for example,
     *   rgb2yuv[] = {
             230, 594, 52,
             -125, -323, 448,
             448, -412, -36,
             66048, 524800,524800
         };
       for BT601 calculation:
       Y=0.299R+0.587G+0.114B
       U=Cb=0.564(B−Y)+128=0.499704B−0.168636R−0.331068G+128(for 10bit, it should 512)
       V=Cr=0.713(R−Y)+128=0.499813R−0.418531G−0.081282B+128(for 10bit, it should 512)

       fixed-point to speed calculate,(for 10bit)
       Y=(0.299R+0.587G+0.114B)*1024/1024=(306*R+601*G+117*B+0)>>10
       U=Cb=0.564(B−Y)+512=0.499704B−0.168636R−0.331068G+512
                          =(−0.168636R−0.331068G+0.499704B+512)*1024/1024
                          =((-173)*R+(-339)*G+512*B+524288)>>10
       V=Cr=0.713(R−Y)+128=0.499813R−0.418531G−0.081282B+512
                          =(0.499813R−0.418531G−0.081282B+512)*1024/1024
                          =512*R+(-429)*G+(-83)*B+52488
       so the table, { 306,601,117,-173,-339,512,512,-429,-83,0,52488,52488}
     */
    VPS_PROPERTY_ARRAY_CSC_RGB_TO_YUV,
    /* set/get source image yuv mode */
    VPS_PROPERTY_YUV_COLOR_MODE_SRC,
    /* set/get destination image yuv mode */
    VPS_PROPERTY_YUV_COLOR_MODE_DST,
    /* set HAE multi-core work mode */
    VPS_PROPERTY_HAE_MULTICORE_MODE,
} VPS_PROPERTY_TYPE_E;

typedef struct esVPS_PROPERTY_S {
    VPS_PROPERTY_TYPE_E type;
    union {
        VPS_RESIZE_METHOD_E resizeMethod;
        ES_S32 cscTable[VPS_CSC_TABLE_SIZE];
        VPS_YUV_COLOR_MODE_E colorMode;
        VPS_HAE_MULTICORE_MODE_E haeMultiCoreMode;
    } u;
} VPS_PROPERTY_S;

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */
#endif /* __ES_COMM_VPS_H__ */
